# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/internal_spark_cluster_status.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/internal_spark_cluster_status.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n5tecton_proto/data/internal_spark_cluster_status.proto\x12\x11tecton_proto.data\"\xc3\x01\n\x1aInternalSparkClusterStatus\x12I\n\x06status\x18\x01 \x01(\x0e\x32\x31.tecton_proto.data.InternalSparkClusterStatusEnumR\x06status\x12\x14\n\x05\x65rror\x18\x02 \x01(\x08R\x05\x65rror\x12#\n\rerror_message\x18\x03 \x01(\tR\x0c\x65rrorMessage\x12\x1f\n\x0b\x63luster_url\x18\x04 \x01(\tR\nclusterUrl*\xc5\x02\n\x1eInternalSparkClusterStatusEnum\x12)\n%INTERNAL_SPARK_CLUSTER_STATUS_UNKNOWN\x10\x00\x12,\n(INTERNAL_SPARK_CLUSTER_STATUS_NO_CLUSTER\x10\x01\x12\x32\n.INTERNAL_SPARK_CLUSTER_STATUS_CREATING_CLUSTER\x10\x02\x12>\n:INTERNAL_SPARK_CLUSTER_STATUS_WAITING_FOR_CLUSTER_TO_START\x10\x03\x12)\n%INTERNAL_SPARK_CLUSTER_STATUS_HEALTHY\x10\x04\x12+\n\'INTERNAL_SPARK_CLUSTER_STATUS_UNHEALTHY\x10\x05\x42\x13\n\x0f\x63om.tecton.dataP\x01'
)

_INTERNALSPARKCLUSTERSTATUSENUM = _descriptor.EnumDescriptor(
  name='InternalSparkClusterStatusEnum',
  full_name='tecton_proto.data.InternalSparkClusterStatusEnum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_NO_CLUSTER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_CREATING_CLUSTER', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_WAITING_FOR_CLUSTER_TO_START', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_HEALTHY', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL_SPARK_CLUSTER_STATUS_UNHEALTHY', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=275,
  serialized_end=600,
)
_sym_db.RegisterEnumDescriptor(_INTERNALSPARKCLUSTERSTATUSENUM)

InternalSparkClusterStatusEnum = enum_type_wrapper.EnumTypeWrapper(_INTERNALSPARKCLUSTERSTATUSENUM)
INTERNAL_SPARK_CLUSTER_STATUS_UNKNOWN = 0
INTERNAL_SPARK_CLUSTER_STATUS_NO_CLUSTER = 1
INTERNAL_SPARK_CLUSTER_STATUS_CREATING_CLUSTER = 2
INTERNAL_SPARK_CLUSTER_STATUS_WAITING_FOR_CLUSTER_TO_START = 3
INTERNAL_SPARK_CLUSTER_STATUS_HEALTHY = 4
INTERNAL_SPARK_CLUSTER_STATUS_UNHEALTHY = 5



_INTERNALSPARKCLUSTERSTATUS = _descriptor.Descriptor(
  name='InternalSparkClusterStatus',
  full_name='tecton_proto.data.InternalSparkClusterStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='tecton_proto.data.InternalSparkClusterStatus.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='tecton_proto.data.InternalSparkClusterStatus.error', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='error', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='tecton_proto.data.InternalSparkClusterStatus.error_message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='errorMessage', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_url', full_name='tecton_proto.data.InternalSparkClusterStatus.cluster_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=77,
  serialized_end=272,
)

_INTERNALSPARKCLUSTERSTATUS.fields_by_name['status'].enum_type = _INTERNALSPARKCLUSTERSTATUSENUM
DESCRIPTOR.message_types_by_name['InternalSparkClusterStatus'] = _INTERNALSPARKCLUSTERSTATUS
DESCRIPTOR.enum_types_by_name['InternalSparkClusterStatusEnum'] = _INTERNALSPARKCLUSTERSTATUSENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InternalSparkClusterStatus = _reflection.GeneratedProtocolMessageType('InternalSparkClusterStatus', (_message.Message,), {
  'DESCRIPTOR' : _INTERNALSPARKCLUSTERSTATUS,
  '__module__' : 'tecton_proto.data.internal_spark_cluster_status_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.InternalSparkClusterStatus)
  })
_sym_db.RegisterMessage(InternalSparkClusterStatus)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
