# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/state_update.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.args import fco_args_pb2 as tecton__proto_dot_args_dot_fco__args__pb2
from tecton_proto.args import repo_metadata_pb2 as tecton__proto_dot_args_dot_repo__metadata__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/state_update.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$tecton_proto/data/state_update.proto\x12\x11tecton_proto.data\x1a\x1ctecton_proto/common/id.proto\x1a tecton_proto/args/fco_args.proto\x1a%tecton_proto/args/repo_metadata.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a$tecton_proto/args/diff_options.proto\"\xd0\x01\n\x0f\x46\x63oPropertyDiff\x12#\n\rproperty_name\x18\x01 \x01(\tR\x0cpropertyName\x12!\n\x0cval_existing\x18\x02 \x01(\tR\x0bvalExisting\x12!\n\x0cval_declared\x18\x03 \x01(\tR\x0bvalDeclared\x12R\n\x0erendering_type\x18\x04 \x01(\x0e\x32+.tecton_proto.args.FcoPropertyRenderingTypeR\rrenderingType\"\xe6\x01\n\x1c\x42\x61tchMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\x12\x46\n\x11schedule_interval\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x10scheduleInterval\x12W\n\x0b\x64\x65stination\x18\x03 \x01(\x0e\x32\x35.tecton_proto.data.MaterializationTaskDiffDestinationR\x0b\x64\x65stination\"F\n\x1dStreamMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\"\xd7\x02\n\x1f\x42\x61\x63kfillMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\x12H\n\x12\x66\x65\x61ture_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12$\n\x0enumber_of_jobs\x18\x04 \x01(\x05R\x0cnumberOfJobs\x12W\n\x0b\x64\x65stination\x18\x05 \x01(\x0e\x32\x35.tecton_proto.data.MaterializationTaskDiffDestinationR\x0b\x64\x65stination\"\xae\x02\n\x13MaterializationInfo\x12\x62\n\x13\x62\x61\x63kfill_task_diffs\x18\x01 \x03(\x0b\x32\x32.tecton_proto.data.BackfillMaterializationTaskDiffR\x11\x62\x61\x63kfillTaskDiffs\x12W\n\x0f\x62\x61tch_task_diff\x18\x02 \x01(\x0b\x32/.tecton_proto.data.BatchMaterializationTaskDiffR\rbatchTaskDiff\x12Z\n\x10stream_task_diff\x18\x03 \x01(\x0b\x32\x30.tecton_proto.data.StreamMaterializationTaskDiffR\x0estreamTaskDiff\"\xde\x02\n\x07\x46\x63oDiff\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32$.tecton_proto.data.FcoTransitionTypeR\x04type\x12?\n\rexisting_args\x18\x07 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x65xistingArgs\x12?\n\rdeclared_args\x18\x08 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x64\x65\x63laredArgs\x12\x36\n\x04\x64iff\x18\x04 \x03(\x0b\x32\".tecton_proto.data.FcoPropertyDiffR\x04\x64iff\x12Y\n\x14materialization_info\x18\x0b \x01(\x0b\x32&.tecton_proto.data.MaterializationInfoR\x13materializationInfoJ\x04\x08\t\x10\n\"C\n\x0b\x46\x63oFieldRef\x12.\n\x06\x66\x63o_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x05\x66\x63oIdJ\x04\x08\x02\x10\x03\"h\n\x11ValidationMessage\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\x12\x39\n\x08\x66\x63o_refs\x18\x02 \x03(\x0b\x32\x1e.tecton_proto.data.FcoFieldRefR\x07\x66\x63oRefs\"\x92\x01\n\x10ValidationResult\x12<\n\x06\x65rrors\x18\x01 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x06\x65rrors\x12@\n\x08warnings\x18\x02 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x08warnings\"\xe3\x02\n\x12StateUpdateRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x35\n\x08\x66\x63o_args\x18\x02 \x03(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x07\x66\x63oArgs\x12R\n\x10repo_source_info\x18\x03 \x01(\x0b\x32(.tecton_proto.args.FeatureRepoSourceInfoR\x0erepoSourceInfo\x12-\n\x12suppress_recreates\x18\n \x01(\x08R\x11suppressRecreates\x12\x1f\n\x0bupgrade_all\x18\x04 \x01(\x08R\nupgradeAll\x12!\n\x0crequested_by\x18\x05 \x01(\tR\x0brequestedBy\x12\x1f\n\x0bsdk_version\x18\t \x01(\tR\nsdkVersionJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t\"\xa7\x01\n\x10StateUpdateEntry\x12\x1b\n\tcommit_id\x18\x01 \x01(\tR\x08\x63ommitId\x12\x1d\n\napplied_by\x18\x02 \x01(\tR\tappliedBy\x12\x39\n\napplied_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tappliedAt\x12\x1c\n\tworkspace\x18\x04 \x01(\tR\tworkspace\"\xb8\x02\n\x16StateUpdatePlanSummary\x12L\n\ndiff_items\x18\x01 \x03(\x0b\x32-.tecton_proto.data.StateUpdatePlanSummaryDiffR\tdiffItems\x12\x1d\n\napplied_by\x18\x02 \x01(\tR\tappliedBy\x12\x1d\n\ncreated_by\x18\x03 \x01(\tR\tcreatedBy\x12\x39\n\napplied_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tappliedAt\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x1c\n\tworkspace\x18\x06 \x01(\tR\tworkspace\"\xbc\x02\n\x1aStateUpdatePlanSummaryDiff\x12\x19\n\x08\x66\x63o_type\x18\x01 \x01(\tR\x07\x66\x63oType\x12\x38\n\x04type\x18\x02 \x01(\x0e\x32$.tecton_proto.data.FcoTransitionTypeR\x04type\x12\x38\n\x05\x64iffs\x18\x03 \x03(\x0b\x32\".tecton_proto.data.FcoPropertyDiffR\x05\x64iffs\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x05 \x01(\tR\x0b\x64\x65scription\x12Y\n\x14materialization_info\x18\x06 \x01(\x0b\x32&.tecton_proto.data.MaterializationInfoR\x13materializationInfo*n\n\x11\x46\x63oTransitionType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\n\n\x06\x44\x45LETE\x10\x02\x12\x0b\n\x07UPGRADE\x10\x05\x12\n\n\x06UPDATE\x10\x03\x12\x0c\n\x08RECREATE\x10\x04\x12\r\n\tUNCHANGED\x10\x06*\xfe\x01\n\"MaterializationTaskDiffDestination\x12\x35\n1MATERIALIZATION_TASK_DIFF_DESTINATION_UNSPECIFIED\x10\x00\x12\x31\n-MATERIALIZATION_TASK_DIFF_DESTINATION_OFFLINE\x10\x01\x12\x30\n,MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE\x10\x02\x12<\n8MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE_AND_OFFLINE\x10\x03\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_fco__args__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_repo__metadata__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_diff__options__pb2.DESCRIPTOR,])

_FCOTRANSITIONTYPE = _descriptor.EnumDescriptor(
  name='FcoTransitionType',
  full_name='tecton_proto.data.FcoTransitionType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPGRADE', index=3, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=4, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RECREATE', index=5, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNCHANGED', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3271,
  serialized_end=3381,
)
_sym_db.RegisterEnumDescriptor(_FCOTRANSITIONTYPE)

FcoTransitionType = enum_type_wrapper.EnumTypeWrapper(_FCOTRANSITIONTYPE)
_MATERIALIZATIONTASKDIFFDESTINATION = _descriptor.EnumDescriptor(
  name='MaterializationTaskDiffDestination',
  full_name='tecton_proto.data.MaterializationTaskDiffDestination',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MATERIALIZATION_TASK_DIFF_DESTINATION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MATERIALIZATION_TASK_DIFF_DESTINATION_OFFLINE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE_AND_OFFLINE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3384,
  serialized_end=3638,
)
_sym_db.RegisterEnumDescriptor(_MATERIALIZATIONTASKDIFFDESTINATION)

MaterializationTaskDiffDestination = enum_type_wrapper.EnumTypeWrapper(_MATERIALIZATIONTASKDIFFDESTINATION)
UNKNOWN = 0
CREATE = 1
DELETE = 2
UPGRADE = 5
UPDATE = 3
RECREATE = 4
UNCHANGED = 6
MATERIALIZATION_TASK_DIFF_DESTINATION_UNSPECIFIED = 0
MATERIALIZATION_TASK_DIFF_DESTINATION_OFFLINE = 1
MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE = 2
MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE_AND_OFFLINE = 3



_FCOPROPERTYDIFF = _descriptor.Descriptor(
  name='FcoPropertyDiff',
  full_name='tecton_proto.data.FcoPropertyDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='property_name', full_name='tecton_proto.data.FcoPropertyDiff.property_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='propertyName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='val_existing', full_name='tecton_proto.data.FcoPropertyDiff.val_existing', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='valExisting', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='val_declared', full_name='tecton_proto.data.FcoPropertyDiff.val_declared', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='valDeclared', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rendering_type', full_name='tecton_proto.data.FcoPropertyDiff.rendering_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='renderingType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=266,
  serialized_end=474,
)


_BATCHMATERIALIZATIONTASKDIFF = _descriptor.Descriptor(
  name='BatchMaterializationTaskDiff',
  full_name='tecton_proto.data.BatchMaterializationTaskDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='display_string', full_name='tecton_proto.data.BatchMaterializationTaskDiff.display_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='displayString', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schedule_interval', full_name='tecton_proto.data.BatchMaterializationTaskDiff.schedule_interval', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scheduleInterval', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination', full_name='tecton_proto.data.BatchMaterializationTaskDiff.destination', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='destination', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=477,
  serialized_end=707,
)


_STREAMMATERIALIZATIONTASKDIFF = _descriptor.Descriptor(
  name='StreamMaterializationTaskDiff',
  full_name='tecton_proto.data.StreamMaterializationTaskDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='display_string', full_name='tecton_proto.data.StreamMaterializationTaskDiff.display_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='displayString', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=709,
  serialized_end=779,
)


_BACKFILLMATERIALIZATIONTASKDIFF = _descriptor.Descriptor(
  name='BackfillMaterializationTaskDiff',
  full_name='tecton_proto.data.BackfillMaterializationTaskDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='display_string', full_name='tecton_proto.data.BackfillMaterializationTaskDiff.display_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='displayString', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_start_time', full_name='tecton_proto.data.BackfillMaterializationTaskDiff.feature_start_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureStartTime', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_end_time', full_name='tecton_proto.data.BackfillMaterializationTaskDiff.feature_end_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureEndTime', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='number_of_jobs', full_name='tecton_proto.data.BackfillMaterializationTaskDiff.number_of_jobs', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='numberOfJobs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='destination', full_name='tecton_proto.data.BackfillMaterializationTaskDiff.destination', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='destination', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=782,
  serialized_end=1125,
)


_MATERIALIZATIONINFO = _descriptor.Descriptor(
  name='MaterializationInfo',
  full_name='tecton_proto.data.MaterializationInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='backfill_task_diffs', full_name='tecton_proto.data.MaterializationInfo.backfill_task_diffs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='backfillTaskDiffs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_task_diff', full_name='tecton_proto.data.MaterializationInfo.batch_task_diff', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='batchTaskDiff', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream_task_diff', full_name='tecton_proto.data.MaterializationInfo.stream_task_diff', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamTaskDiff', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1128,
  serialized_end=1430,
)


_FCODIFF = _descriptor.Descriptor(
  name='FcoDiff',
  full_name='tecton_proto.data.FcoDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.data.FcoDiff.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='existing_args', full_name='tecton_proto.data.FcoDiff.existing_args', index=1,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='existingArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='declared_args', full_name='tecton_proto.data.FcoDiff.declared_args', index=2,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='declaredArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='diff', full_name='tecton_proto.data.FcoDiff.diff', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='diff', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='materialization_info', full_name='tecton_proto.data.FcoDiff.materialization_info', index=4,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='materializationInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1433,
  serialized_end=1783,
)


_FCOFIELDREF = _descriptor.Descriptor(
  name='FcoFieldRef',
  full_name='tecton_proto.data.FcoFieldRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fco_id', full_name='tecton_proto.data.FcoFieldRef.fco_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1785,
  serialized_end=1852,
)


_VALIDATIONMESSAGE = _descriptor.Descriptor(
  name='ValidationMessage',
  full_name='tecton_proto.data.ValidationMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='tecton_proto.data.ValidationMessage.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='message', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fco_refs', full_name='tecton_proto.data.ValidationMessage.fco_refs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoRefs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1854,
  serialized_end=1958,
)


_VALIDATIONRESULT = _descriptor.Descriptor(
  name='ValidationResult',
  full_name='tecton_proto.data.ValidationResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='errors', full_name='tecton_proto.data.ValidationResult.errors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='errors', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='warnings', full_name='tecton_proto.data.ValidationResult.warnings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='warnings', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1961,
  serialized_end=2107,
)


_STATEUPDATEREQUEST = _descriptor.Descriptor(
  name='StateUpdateRequest',
  full_name='tecton_proto.data.StateUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.StateUpdateRequest.workspace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fco_args', full_name='tecton_proto.data.StateUpdateRequest.fco_args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repo_source_info', full_name='tecton_proto.data.StateUpdateRequest.repo_source_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='repoSourceInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='suppress_recreates', full_name='tecton_proto.data.StateUpdateRequest.suppress_recreates', index=3,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='suppressRecreates', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upgrade_all', full_name='tecton_proto.data.StateUpdateRequest.upgrade_all', index=4,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='upgradeAll', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requested_by', full_name='tecton_proto.data.StateUpdateRequest.requested_by', index=5,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='requestedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sdk_version', full_name='tecton_proto.data.StateUpdateRequest.sdk_version', index=6,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sdkVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2110,
  serialized_end=2465,
)


_STATEUPDATEENTRY = _descriptor.Descriptor(
  name='StateUpdateEntry',
  full_name='tecton_proto.data.StateUpdateEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='commit_id', full_name='tecton_proto.data.StateUpdateEntry.commit_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='commitId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_by', full_name='tecton_proto.data.StateUpdateEntry.applied_by', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_at', full_name='tecton_proto.data.StateUpdateEntry.applied_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.StateUpdateEntry.workspace', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2468,
  serialized_end=2635,
)


_STATEUPDATEPLANSUMMARY = _descriptor.Descriptor(
  name='StateUpdatePlanSummary',
  full_name='tecton_proto.data.StateUpdatePlanSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='diff_items', full_name='tecton_proto.data.StateUpdatePlanSummary.diff_items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='diffItems', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_by', full_name='tecton_proto.data.StateUpdatePlanSummary.applied_by', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_by', full_name='tecton_proto.data.StateUpdatePlanSummary.created_by', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='createdBy', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='applied_at', full_name='tecton_proto.data.StateUpdatePlanSummary.applied_at', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='appliedAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='tecton_proto.data.StateUpdatePlanSummary.created_at', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='createdAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.StateUpdatePlanSummary.workspace', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2638,
  serialized_end=2950,
)


_STATEUPDATEPLANSUMMARYDIFF = _descriptor.Descriptor(
  name='StateUpdatePlanSummaryDiff',
  full_name='tecton_proto.data.StateUpdatePlanSummaryDiff',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fco_type', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.fco_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fcoType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='diffs', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.diffs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='diffs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='description', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='materialization_info', full_name='tecton_proto.data.StateUpdatePlanSummaryDiff.materialization_info', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='materializationInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2953,
  serialized_end=3269,
)

_FCOPROPERTYDIFF.fields_by_name['rendering_type'].enum_type = tecton__proto_dot_args_dot_diff__options__pb2._FCOPROPERTYRENDERINGTYPE
_BATCHMATERIALIZATIONTASKDIFF.fields_by_name['schedule_interval'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_BATCHMATERIALIZATIONTASKDIFF.fields_by_name['destination'].enum_type = _MATERIALIZATIONTASKDIFFDESTINATION
_BACKFILLMATERIALIZATIONTASKDIFF.fields_by_name['feature_start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_BACKFILLMATERIALIZATIONTASKDIFF.fields_by_name['feature_end_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_BACKFILLMATERIALIZATIONTASKDIFF.fields_by_name['destination'].enum_type = _MATERIALIZATIONTASKDIFFDESTINATION
_MATERIALIZATIONINFO.fields_by_name['backfill_task_diffs'].message_type = _BACKFILLMATERIALIZATIONTASKDIFF
_MATERIALIZATIONINFO.fields_by_name['batch_task_diff'].message_type = _BATCHMATERIALIZATIONTASKDIFF
_MATERIALIZATIONINFO.fields_by_name['stream_task_diff'].message_type = _STREAMMATERIALIZATIONTASKDIFF
_FCODIFF.fields_by_name['type'].enum_type = _FCOTRANSITIONTYPE
_FCODIFF.fields_by_name['existing_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_FCODIFF.fields_by_name['declared_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_FCODIFF.fields_by_name['diff'].message_type = _FCOPROPERTYDIFF
_FCODIFF.fields_by_name['materialization_info'].message_type = _MATERIALIZATIONINFO
_FCOFIELDREF.fields_by_name['fco_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_VALIDATIONMESSAGE.fields_by_name['fco_refs'].message_type = _FCOFIELDREF
_VALIDATIONRESULT.fields_by_name['errors'].message_type = _VALIDATIONMESSAGE
_VALIDATIONRESULT.fields_by_name['warnings'].message_type = _VALIDATIONMESSAGE
_STATEUPDATEREQUEST.fields_by_name['fco_args'].message_type = tecton__proto_dot_args_dot_fco__args__pb2._FCOARGS
_STATEUPDATEREQUEST.fields_by_name['repo_source_info'].message_type = tecton__proto_dot_args_dot_repo__metadata__pb2._FEATUREREPOSOURCEINFO
_STATEUPDATEENTRY.fields_by_name['applied_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STATEUPDATEPLANSUMMARY.fields_by_name['diff_items'].message_type = _STATEUPDATEPLANSUMMARYDIFF
_STATEUPDATEPLANSUMMARY.fields_by_name['applied_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STATEUPDATEPLANSUMMARY.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STATEUPDATEPLANSUMMARYDIFF.fields_by_name['type'].enum_type = _FCOTRANSITIONTYPE
_STATEUPDATEPLANSUMMARYDIFF.fields_by_name['diffs'].message_type = _FCOPROPERTYDIFF
_STATEUPDATEPLANSUMMARYDIFF.fields_by_name['materialization_info'].message_type = _MATERIALIZATIONINFO
DESCRIPTOR.message_types_by_name['FcoPropertyDiff'] = _FCOPROPERTYDIFF
DESCRIPTOR.message_types_by_name['BatchMaterializationTaskDiff'] = _BATCHMATERIALIZATIONTASKDIFF
DESCRIPTOR.message_types_by_name['StreamMaterializationTaskDiff'] = _STREAMMATERIALIZATIONTASKDIFF
DESCRIPTOR.message_types_by_name['BackfillMaterializationTaskDiff'] = _BACKFILLMATERIALIZATIONTASKDIFF
DESCRIPTOR.message_types_by_name['MaterializationInfo'] = _MATERIALIZATIONINFO
DESCRIPTOR.message_types_by_name['FcoDiff'] = _FCODIFF
DESCRIPTOR.message_types_by_name['FcoFieldRef'] = _FCOFIELDREF
DESCRIPTOR.message_types_by_name['ValidationMessage'] = _VALIDATIONMESSAGE
DESCRIPTOR.message_types_by_name['ValidationResult'] = _VALIDATIONRESULT
DESCRIPTOR.message_types_by_name['StateUpdateRequest'] = _STATEUPDATEREQUEST
DESCRIPTOR.message_types_by_name['StateUpdateEntry'] = _STATEUPDATEENTRY
DESCRIPTOR.message_types_by_name['StateUpdatePlanSummary'] = _STATEUPDATEPLANSUMMARY
DESCRIPTOR.message_types_by_name['StateUpdatePlanSummaryDiff'] = _STATEUPDATEPLANSUMMARYDIFF
DESCRIPTOR.enum_types_by_name['FcoTransitionType'] = _FCOTRANSITIONTYPE
DESCRIPTOR.enum_types_by_name['MaterializationTaskDiffDestination'] = _MATERIALIZATIONTASKDIFFDESTINATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FcoPropertyDiff = _reflection.GeneratedProtocolMessageType('FcoPropertyDiff', (_message.Message,), {
  'DESCRIPTOR' : _FCOPROPERTYDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoPropertyDiff)
  })
_sym_db.RegisterMessage(FcoPropertyDiff)

BatchMaterializationTaskDiff = _reflection.GeneratedProtocolMessageType('BatchMaterializationTaskDiff', (_message.Message,), {
  'DESCRIPTOR' : _BATCHMATERIALIZATIONTASKDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.BatchMaterializationTaskDiff)
  })
_sym_db.RegisterMessage(BatchMaterializationTaskDiff)

StreamMaterializationTaskDiff = _reflection.GeneratedProtocolMessageType('StreamMaterializationTaskDiff', (_message.Message,), {
  'DESCRIPTOR' : _STREAMMATERIALIZATIONTASKDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StreamMaterializationTaskDiff)
  })
_sym_db.RegisterMessage(StreamMaterializationTaskDiff)

BackfillMaterializationTaskDiff = _reflection.GeneratedProtocolMessageType('BackfillMaterializationTaskDiff', (_message.Message,), {
  'DESCRIPTOR' : _BACKFILLMATERIALIZATIONTASKDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.BackfillMaterializationTaskDiff)
  })
_sym_db.RegisterMessage(BackfillMaterializationTaskDiff)

MaterializationInfo = _reflection.GeneratedProtocolMessageType('MaterializationInfo', (_message.Message,), {
  'DESCRIPTOR' : _MATERIALIZATIONINFO,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.MaterializationInfo)
  })
_sym_db.RegisterMessage(MaterializationInfo)

FcoDiff = _reflection.GeneratedProtocolMessageType('FcoDiff', (_message.Message,), {
  'DESCRIPTOR' : _FCODIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoDiff)
  })
_sym_db.RegisterMessage(FcoDiff)

FcoFieldRef = _reflection.GeneratedProtocolMessageType('FcoFieldRef', (_message.Message,), {
  'DESCRIPTOR' : _FCOFIELDREF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.FcoFieldRef)
  })
_sym_db.RegisterMessage(FcoFieldRef)

ValidationMessage = _reflection.GeneratedProtocolMessageType('ValidationMessage', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONMESSAGE,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ValidationMessage)
  })
_sym_db.RegisterMessage(ValidationMessage)

ValidationResult = _reflection.GeneratedProtocolMessageType('ValidationResult', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONRESULT,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ValidationResult)
  })
_sym_db.RegisterMessage(ValidationResult)

StateUpdateRequest = _reflection.GeneratedProtocolMessageType('StateUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEREQUEST,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdateRequest)
  })
_sym_db.RegisterMessage(StateUpdateRequest)

StateUpdateEntry = _reflection.GeneratedProtocolMessageType('StateUpdateEntry', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEENTRY,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdateEntry)
  })
_sym_db.RegisterMessage(StateUpdateEntry)

StateUpdatePlanSummary = _reflection.GeneratedProtocolMessageType('StateUpdatePlanSummary', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEPLANSUMMARY,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdatePlanSummary)
  })
_sym_db.RegisterMessage(StateUpdatePlanSummary)

StateUpdatePlanSummaryDiff = _reflection.GeneratedProtocolMessageType('StateUpdatePlanSummaryDiff', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATEPLANSUMMARYDIFF,
  '__module__' : 'tecton_proto.data.state_update_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StateUpdatePlanSummaryDiff)
  })
_sym_db.RegisterMessage(StateUpdatePlanSummaryDiff)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
