# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/framework_version.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/common/framework_version.proto',
  package='tecton_proto.common',
  syntax='proto2',
  serialized_options=b'\n\021com.tecton.commonP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n+tecton_proto/common/framework_version.proto\x12\x13tecton_proto.common*M\n\x10\x46rameworkVersion\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04\x46WV3\x10\x03\x12\x08\n\x04\x46WV4\x10\x04\x12\x08\n\x04\x46WV5\x10\x05\"\x04\x08\x01\x10\x01\"\x04\x08\x02\x10\x02\x42\x15\n\x11\x63om.tecton.commonP\x01'
)

_FRAMEWORKVERSION = _descriptor.EnumDescriptor(
  name='FrameworkVersion',
  full_name='tecton_proto.common.FrameworkVersion',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FWV3', index=1, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FWV4', index=2, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FWV5', index=3, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=68,
  serialized_end=145,
)
_sym_db.RegisterEnumDescriptor(_FRAMEWORKVERSION)

FrameworkVersion = enum_type_wrapper.EnumTypeWrapper(_FRAMEWORKVERSION)
UNSPECIFIED = 0
FWV3 = 3
FWV4 = 4
FWV5 = 5


DESCRIPTOR.enum_types_by_name['FrameworkVersion'] = _FRAMEWORKVERSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
