# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/version_constraints.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/version_constraints.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n+tecton_proto/args/version_constraints.proto\x12\x11tecton_proto.args\x1a google/protobuf/descriptor.proto\x1a+tecton_proto/common/framework_version.proto\"\xad\x01\n\x1b\x46rameworkVersionConstraints\x12\x46\n\x0bmin_version\x18\x01 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionR\nminVersion\x12\x46\n\x0bmax_version\x18\x02 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionR\nmaxVersion:{\n\x11\x66ramework_version\x12\x1d.google.protobuf.FieldOptions\x18\xd0\x0f \x01(\x0b\x32..tecton_proto.args.FrameworkVersionConstraintsR\x10\x66rameworkVersionB\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_framework__version__pb2.DESCRIPTOR,])


FRAMEWORK_VERSION_FIELD_NUMBER = 2000
framework_version = _descriptor.FieldDescriptor(
  name='framework_version', full_name='tecton_proto.args.framework_version', index=0,
  number=2000, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, json_name='frameworkVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key)


_FRAMEWORKVERSIONCONSTRAINTS = _descriptor.Descriptor(
  name='FrameworkVersionConstraints',
  full_name='tecton_proto.args.FrameworkVersionConstraints',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_version', full_name='tecton_proto.args.FrameworkVersionConstraints.min_version', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='minVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_version', full_name='tecton_proto.args.FrameworkVersionConstraints.max_version', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='maxVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=146,
  serialized_end=319,
)

_FRAMEWORKVERSIONCONSTRAINTS.fields_by_name['min_version'].enum_type = tecton__proto_dot_common_dot_framework__version__pb2._FRAMEWORKVERSION
_FRAMEWORKVERSIONCONSTRAINTS.fields_by_name['max_version'].enum_type = tecton__proto_dot_common_dot_framework__version__pb2._FRAMEWORKVERSION
DESCRIPTOR.message_types_by_name['FrameworkVersionConstraints'] = _FRAMEWORKVERSIONCONSTRAINTS
DESCRIPTOR.extensions_by_name['framework_version'] = framework_version
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FrameworkVersionConstraints = _reflection.GeneratedProtocolMessageType('FrameworkVersionConstraints', (_message.Message,), {
  'DESCRIPTOR' : _FRAMEWORKVERSIONCONSTRAINTS,
  '__module__' : 'tecton_proto.args.version_constraints_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.FrameworkVersionConstraints)
  })
_sym_db.RegisterMessage(FrameworkVersionConstraints)

framework_version.message_type = _FRAMEWORKVERSIONCONSTRAINTS
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(framework_version)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
