# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/onboarding.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/onboarding.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"tecton_proto/data/onboarding.proto\x12\x11tecton_proto.data\x1a\x1ctecton_proto/common/id.proto\"l\n\x10OnboardingStatus\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12?\n\x06status\x18\x02 \x01(\x0e\x32\'.tecton_proto.data.OnboardingStatusEnumR\x06status\"\xdd\x01\n\x1b\x44\x61taPlatformSetupTaskStatus\x12*\n\x11task_display_name\x18\x01 \x01(\tR\x0ftaskDisplayName\x12S\n\x0btask_status\x18\x02 \x01(\x0e\x32\x32.tecton_proto.data.DataPlatformSetupTaskStatusEnumR\ntaskStatus\x12#\n\rerror_message\x18\x03 \x01(\tR\x0c\x65rrorMessage\x12\x18\n\x07\x64\x65tails\x18\x04 \x01(\tR\x07\x64\x65tails*x\n\x14OnboardingStatusEnum\x12\x1d\n\x19ONBOARDING_STATUS_UNKNOWN\x10\x00\x12 \n\x1cONBOARDING_STATUS_INCOMPLETE\x10\x01\x12\x1f\n\x1bONBOARDING_STATUS_COMPLETED\x10\x02*\xa3\x01\n\x1f\x44\x61taPlatformSetupTaskStatusEnum\x12\x17\n\x13TASK_STATUS_UNKNOWN\x10\x00\x12\x1b\n\x17TASK_STATUS_NOT_STARTED\x10\x01\x12\x17\n\x13TASK_STATUS_RUNNING\x10\x02\x12\x19\n\x15TASK_STATUS_SUCCEEDED\x10\x03\x12\x16\n\x12TASK_STATUS_FAILED\x10\x04\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,])

_ONBOARDINGSTATUSENUM = _descriptor.EnumDescriptor(
  name='OnboardingStatusEnum',
  full_name='tecton_proto.data.OnboardingStatusEnum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ONBOARDING_STATUS_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONBOARDING_STATUS_INCOMPLETE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONBOARDING_STATUS_COMPLETED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=421,
  serialized_end=541,
)
_sym_db.RegisterEnumDescriptor(_ONBOARDINGSTATUSENUM)

OnboardingStatusEnum = enum_type_wrapper.EnumTypeWrapper(_ONBOARDINGSTATUSENUM)
_DATAPLATFORMSETUPTASKSTATUSENUM = _descriptor.EnumDescriptor(
  name='DataPlatformSetupTaskStatusEnum',
  full_name='tecton_proto.data.DataPlatformSetupTaskStatusEnum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TASK_STATUS_UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TASK_STATUS_NOT_STARTED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TASK_STATUS_RUNNING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TASK_STATUS_SUCCEEDED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TASK_STATUS_FAILED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=544,
  serialized_end=707,
)
_sym_db.RegisterEnumDescriptor(_DATAPLATFORMSETUPTASKSTATUSENUM)

DataPlatformSetupTaskStatusEnum = enum_type_wrapper.EnumTypeWrapper(_DATAPLATFORMSETUPTASKSTATUSENUM)
ONBOARDING_STATUS_UNKNOWN = 0
ONBOARDING_STATUS_INCOMPLETE = 1
ONBOARDING_STATUS_COMPLETED = 2
TASK_STATUS_UNKNOWN = 0
TASK_STATUS_NOT_STARTED = 1
TASK_STATUS_RUNNING = 2
TASK_STATUS_SUCCEEDED = 3
TASK_STATUS_FAILED = 4



_ONBOARDINGSTATUS = _descriptor.Descriptor(
  name='OnboardingStatus',
  full_name='tecton_proto.data.OnboardingStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='tecton_proto.data.OnboardingStatus.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='tecton_proto.data.OnboardingStatus.status', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=195,
)


_DATAPLATFORMSETUPTASKSTATUS = _descriptor.Descriptor(
  name='DataPlatformSetupTaskStatus',
  full_name='tecton_proto.data.DataPlatformSetupTaskStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='task_display_name', full_name='tecton_proto.data.DataPlatformSetupTaskStatus.task_display_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='taskDisplayName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='task_status', full_name='tecton_proto.data.DataPlatformSetupTaskStatus.task_status', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='taskStatus', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='tecton_proto.data.DataPlatformSetupTaskStatus.error_message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='errorMessage', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='details', full_name='tecton_proto.data.DataPlatformSetupTaskStatus.details', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='details', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=198,
  serialized_end=419,
)

_ONBOARDINGSTATUS.fields_by_name['status'].enum_type = _ONBOARDINGSTATUSENUM
_DATAPLATFORMSETUPTASKSTATUS.fields_by_name['task_status'].enum_type = _DATAPLATFORMSETUPTASKSTATUSENUM
DESCRIPTOR.message_types_by_name['OnboardingStatus'] = _ONBOARDINGSTATUS
DESCRIPTOR.message_types_by_name['DataPlatformSetupTaskStatus'] = _DATAPLATFORMSETUPTASKSTATUS
DESCRIPTOR.enum_types_by_name['OnboardingStatusEnum'] = _ONBOARDINGSTATUSENUM
DESCRIPTOR.enum_types_by_name['DataPlatformSetupTaskStatusEnum'] = _DATAPLATFORMSETUPTASKSTATUSENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

OnboardingStatus = _reflection.GeneratedProtocolMessageType('OnboardingStatus', (_message.Message,), {
  'DESCRIPTOR' : _ONBOARDINGSTATUS,
  '__module__' : 'tecton_proto.data.onboarding_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.OnboardingStatus)
  })
_sym_db.RegisterMessage(OnboardingStatus)

DataPlatformSetupTaskStatus = _reflection.GeneratedProtocolMessageType('DataPlatformSetupTaskStatus', (_message.Message,), {
  'DESCRIPTOR' : _DATAPLATFORMSETUPTASKSTATUS,
  '__module__' : 'tecton_proto.data.onboarding_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.DataPlatformSetupTaskStatus)
  })
_sym_db.RegisterMessage(DataPlatformSetupTaskStatus)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
